#include "temperaturegraph.h"
#include <QPaintEvent>
#include <QPainter>
#include <QTime>

TemperatureGraph::TemperatureGraph(QWidget *parent) :
    QWidget(parent)
{
}

void TemperatureGraph::addTemperatureSample(double extruder_temperature, double bed_temperature)
{
    const double time = QTime().msecsTo(QTime::currentTime()) * 1.0e-3;
    extruder_temperature_samples.push_back(std::make_pair(time, extruder_temperature));
    bed_temperature_samples.push_back(std::make_pair(time, bed_temperature));

    update();
}

void TemperatureGraph::paintEvent(QPaintEvent *)
{
    const double time = QTime().msecsTo(QTime::currentTime()) * 1.0e-3;
    const double start_time = time - width();
    const double h = height();
    const double hs = h / 300.0;

    double extruder_temperature = 0.0, bed_temperature = 0.0;
    if (!extruder_temperature_samples.empty())
        extruder_temperature = extruder_temperature_samples.back().second;
    if (!bed_temperature_samples.empty())
        bed_temperature = bed_temperature_samples.back().second;

    QPainter painter(this);
    painter.fillRect(rect(), palette().base());
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::TextAntialiasing);

    painter.setPen(QPen(Qt::blue));
    for(int i = 0 ; i <= 300 ; i += 100)
    {
        painter.drawLine(QLineF(0, h - i * hs, time - start_time, h - i * hs));
        painter.drawText(75, i == 0 ? (h - hs *i - 20) : h - hs *i, QString::fromUtf8("%1°C").arg(i));
    }

    while(extruder_temperature_samples.size() >= 2
          && extruder_temperature_samples[0].first < start_time
          && extruder_temperature_samples[1].first < start_time)
    {
        extruder_temperature_samples.pop_front();
        bed_temperature_samples.pop_front();
    }

    QPen pen(Qt::red);
    pen.setStyle(Qt::SolidLine);
    painter.setPen(pen);
    std::deque<std::pair<double, double> >::const_reverse_iterator prev = extruder_temperature_samples.rend();
    for(std::deque<std::pair<double, double> >::const_reverse_iterator it = extruder_temperature_samples.rbegin() ; it != extruder_temperature_samples.rend() ; ++it)
    {
        if (prev != extruder_temperature_samples.rend())
        {
            if (prev->first < start_time)
                break;
            if (prev->second < 500 && it->second < 500)
            {
                painter.drawLine(QLineF(prev->first - start_time,
                                        h - prev->second * hs,
                                        it->first - start_time,
                                        h - it->second * hs));
            }
        }
        prev = it;
    }
    painter.setPen(QPen(Qt::black));
    painter.drawLine(0, h - extruder_temperature * hs, time - start_time, h - extruder_temperature * hs);
    painter.drawText(75, h - hs * extruder_temperature, QString::fromUtf8("%1°C").arg(extruder_temperature));

    pen.setColor(Qt::blue);
    painter.setPen(pen);
    prev = bed_temperature_samples.rend();
    for(std::deque<std::pair<double, double> >::const_reverse_iterator it = bed_temperature_samples.rbegin() ; it != bed_temperature_samples.rend() ; ++it)
    {
        if (prev != bed_temperature_samples.rend())
        {
            if (prev->first < start_time)
                break;
            if (prev->second < 500 && it->second < 500)
            {
                painter.drawLine(QLineF(prev->first - start_time,
                                        h - prev->second * hs,
                                        it->first - start_time,
                                        h - it->second * hs));
            }
        }
        prev = it;
    }
    painter.setPen(QPen(Qt::black));
    painter.drawLine(QLineF(0, h - bed_temperature * hs, time - start_time, h - bed_temperature * hs));
    painter.drawText(75, h - hs * bed_temperature, QString::fromUtf8("%1°C").arg(bed_temperature));
}
